@@ -1,11 +1,12 @@
 import sys
 import os
-import pipes
 import tomli
+import pkg_resources
+from warnings import warn
+from pipes import quote
 from configparser import ConfigParser, SectionProxy
 from enum import Enum, auto
 from itertools import chain
-import pkg_resources
 from typing import Dict, Iterable, Set, Tuple, List, IO, Union, Callable, Any
 from packaging.utils import canonicalize_name
 from packaging.requirements import InvalidRequirement, Requirement
@@ -60,6 +61,18 @@ def is_configuration_file(path: str) -> bool:
     return True
 
 
+def refresh_working_set() -> None:
+    """
+    Force a refresh of all distribution information and clear related caches
+    """
+    get_installed_distributions.cache_clear()
+    is_editable.cache_clear()
+    is_installed.cache_clear()
+    get_requirement_string_distribution_name.cache_clear()
+    pkg_resources.working_set.entries = []
+    pkg_resources.working_set.__init__()  # type: ignore
+
+
 @lru_cache()
 def get_installed_distributions() -> Dict[str, pkg_resources.Distribution]:
     """
@@ -75,6 +88,11 @@ def get_distribution(name: str) -> pkg_resources.Distribution:
     return get_installed_distributions()[normalize_name(name)]
 
 
+@lru_cache()
+def is_installed(distribution_name: str) -> bool:
+    return normalize_name(distribution_name) in get_installed_distributions()
+
+
 def get_requirement_distribution_name(requirement: Requirement) -> str:
     return normalize_name(requirement.name)
 
@@ -133,16 +151,29 @@ def _iter_tox_ini_requirement_strings(path: str) -> Iterable[str]:
     parser: ConfigParser = ConfigParser()
     parser.read(path)
 
-    def get_section_deps(section_name: str) -> Iterable[str]:
-        if parser.has_option(section_name, "deps"):
+    def get_section_option_requirements(
+        section_name: str, option_name: str
+    ) -> Iterable[str]:
+        if parser.has_option(section_name, option_name):
             return filter(
                 is_requirement_string,
-                parser.get(section_name, "deps").split("\n"),
+                parser.get(section_name, option_name).split("\n"),
             )
         return ()
 
+    def get_section_requirements(section_name: str) -> Iterable[str]:
+        requirements: Iterable[str] = get_section_option_requirements(
+            section_name, "deps"
+        )
+        if section_name == "tox":
+            requirements = chain(
+                requirements,
+                get_section_option_requirements(section_name, "requires"),
+            )
+        return requirements
+
     return unique_everseen(
-        chain(("tox",), *map(get_section_deps, parser.sections()))
+        chain(("tox",), *map(get_section_requirements, parser.sections()))
     )
 
 
@@ -247,14 +278,35 @@ def _iter_editable_distributions(
 def _reinstall_distribution(
     distribution: pkg_resources.Distribution, echo: bool = False
 ) -> None:
-    run(
-        (
-            f"{pipes.quote(sys.executable)} -m pip install --no-deps "
-            f"-e {pipes.quote(distribution.location)}"
-        ),
-        echo=echo,
-    )
-    _reinstalled_locations.add(os.path.abspath(distribution.location))
+    _reinstall_location(distribution.location, echo=echo)
+
+
+def _reinstall_location(location: str, echo: bool = False) -> bool:
+    try:
+        run(
+            (
+                f"{quote(sys.executable)} -m pip install --no-deps "
+                f"-e {quote(location)}"
+            ),
+            echo=echo,
+        )
+        _reinstalled_locations.add(os.path.abspath(location))
+        return True
+    except OSError:
+        # If an error code is returned, we just assume package metadata is
+        # up-to-date
+        return False
+
+
+def reinstall_location(location: str, echo: bool = False) -> str:
+    """
+    Re-install a distribution at the local file path `location`,
+    and return the distribution name, or "" if no distribution could
+    be found
+    """
+    if _reinstall_location(location, echo):
+        return _get_location_distribution_name(location)
+    return ""
 
 
 def reinstall_editable(
@@ -312,17 +364,104 @@ def reinstall_editable(
             ),
         )
     )
-    get_installed_distributions.cache_clear()
+    refresh_working_set()
 
 
-def get_location_distribution_name(location: str) -> str:
+def _get_setup_py_distribution_name(path: str) -> str:
+    current_directory: str = os.curdir
+    if os.path.basename(path).lower() == "setup.py":
+        os.chdir(os.path.dirname(path))
+    else:
+        if not os.path.isdir(path):
+            path = os.path.dirname(path)
+        os.chdir(path)
+        path = os.path.join(path, "setup.py")
+    try:
+        name: str = (
+            run(f"{quote(sys.executable)} {quote(path)} --name", echo=False)
+            .strip()
+            .split("\n")[-1]
+        )
+    except OSError:
+        name = ""
+    os.chdir(current_directory)
+    return name
+
+
+def _get_setup_cfg_distribution_name(path: str) -> str:
+    if os.path.basename(path).lower() != "setup.cfg":
+        if not os.path.isdir(path):
+            path = os.path.dirname(path)
+        path = os.path.join(path, "setup.cfg")
+    if os.path.isfile(path):
+        parser: ConfigParser = ConfigParser()
+        parser.read(path)
+        if "metadata" in parser:
+            return parser.get("metadata", "name", fallback="")
+    return ""
+
+
+def get_setup_distribution_name(path: str) -> str:
     """
-    Get a distribution name based on an installation location
+    Get a distribution's name from setup.py or setup.cfg
     """
-    return _get_location_distribution_name(os.path.abspath(location))
+    return normalize_name(
+        _get_setup_py_distribution_name(path)
+        or _get_setup_cfg_distribution_name(path)
+    )
+
+
+def _setup_dist_egg_info(location: str) -> None:
+    setup_py_path: str = os.path.join(location, "setup.py")
+    # If there is no setup.py file, we can't update egg info
+    if not os.path.isfile(setup_py_path):
+        return
+    current_directory: str = os.curdir
+    os.chdir(location)
+    try:
+        run(
+            (
+                f"{quote(sys.executable)} {quote(setup_py_path)} -q "
+                "dist_info egg_info"
+            ),
+            echo=False,
+        )
+    except OSError:
+        pass
+    os.chdir(current_directory)
+
+
+def setup_dist_egg_info(directory: str) -> None:
+    """
+    Refresh dist-info and egg-info for the editable package installed in
+    `directory`
+    """
+    directory = os.path.abspath(directory)
+    if not os.path.isdir(directory):
+        directory = os.path.dirname(directory)
+    return _setup_dist_egg_info(directory)
+
+
+def get_setup_distribution_requirements(
+    path: str,
+) -> Dict[str, Tuple[str, ...]]:
+    """
+    Get a distribution's name from setup.py or setup.cfg
+    """
+    return normalize_name(
+        _get_setup_py_distribution_name(path)
+        or _get_setup_cfg_distribution_name(path)
+    )
 
 
 def _get_location_distribution_name(location: str) -> str:
+    """
+    Get a distribution name based on an installation location, or return
+    an empty string if no distribution can be found
+    """
+    _setup_dist_egg_info(location)
+    location = os.path.abspath(location)
+
     def _is_in_location(
         name_distribution: Tuple[str, pkg_resources.Distribution]
     ) -> bool:
@@ -333,12 +472,15 @@ def _get_location_distribution_name(location: str) -> str:
     ) -> str:
         return name_distribution[0]
 
-    return next(
-        map(
-            _get_name,
-            filter(_is_in_location, get_installed_distributions().items()),
+    try:
+        return next(
+            map(
+                _get_name,
+                filter(_is_in_location, get_installed_distributions().items()),
+            )
         )
-    )
+    except StopIteration:
+        return get_setup_distribution_name(location)
 
 
 def _get_pkg_requirement(
@@ -375,7 +517,7 @@ def _get_requirement(
             pkg_resources, "extern"
         ).packaging.requirements.InvalidRequirement,
         getattr(pkg_resources, "RequirementParseError"),
-    ) as error:
+    ):
         # Try to parse the requirement as an installation target location,
         # such as can be used with `pip install`
         location: str = requirement_string
@@ -385,11 +527,9 @@ def _get_requirement(
             location = "[".join(parts[:-1])
             extras = f"[{parts[-1]}"
         location = os.path.abspath(location)
-        try:
-            name: str = _get_location_distribution_name(location)
-            return constructor(f"{name}{extras}")
-        except TypeError:
-            raise error
+        name: str = _get_location_distribution_name(location)
+        assert name, f"No distribution found in {location}"
+        return constructor(f"{name}{extras}")
 
 
 def get_required_distribution_names(
@@ -433,13 +573,30 @@ def _iter_requirement_names(
     exclude: Set[str],
     recursive: bool = True,
 ) -> Iterable[str]:
-    project_name: str = normalize_name(requirement.project_name)
+    name: str = normalize_name(requirement.project_name)
     extras: Set[str] = set(map(normalize_name, requirement.extras))
-    if project_name in exclude:
+    if name in exclude:
         return ()
-    distribution: pkg_resources.Distribution = get_installed_distributions()[
-        project_name
-    ]
+    distribution: pkg_resources.Distribution
+    try:
+        distribution = get_installed_distributions()[name]
+    except KeyError:
+        warn(
+            'The distribution "{name}" was not installed, '
+            "attempting to install it now..."
+        )
+        run(
+            (
+                f"{quote(sys.executable)} -m pip install "
+                f"{quote(str(requirement))}"
+            ),
+            echo=True,
+        )
+        refresh_working_set()
+        return _iter_requirement_names(requirement, exclude, recursive)
+    # Ensure requirements are up-to-date
+    if _distribution_is_editable(distribution):
+        _setup_dist_egg_info(distribution.location)
     requirements: List[pkg_resources.Requirement] = distribution.requires(
         extras=tuple(sorted(extras))
     )
@@ -462,3 +619,70 @@ def _iter_requirement_names(
             requirement_names, *map(iter_requirement_names_, requirements)
         )
     return requirement_names
+
+
+def _iter_requirement_strings_required_distribution_names(
+    requirement_strings: Iterable[str],
+) -> Iterable[str]:
+    visited_requirement_strings: Set[str] = set()
+    if isinstance(requirement_strings, str):
+        requirement_strings = (requirement_strings,)
+
+    def get_required_distribution_names_(requirement_string: str) -> Set[str]:
+        if requirement_string not in visited_requirement_strings:
+            try:
+                name: str = get_requirement_string_distribution_name(
+                    requirement_string
+                )
+                visited_requirement_strings.add(requirement_string)
+                return get_required_distribution_names(requirement_string) | {
+                    name
+                }
+            except KeyError:
+                pass
+        return set()
+
+    return unique_everseen(
+        chain(*map(get_required_distribution_names_, requirement_strings)),
+    )
+
+
+def get_requirements_required_distribution_names(
+    requirements: Iterable[str] = (),
+) -> Set[str]:
+    """
+    Get the distributions required by one or more specified distributions or
+    configuration files.
+
+    Parameters:
+
+    - requirements ([str]): One or more requirement specifiers (for example:
+      "requirement-name[extra-a,extra-b]" or ".[extra-a, extra-b]) and/or paths
+      to a setup.cfg, pyproject.toml, tox.ini or requirements.txt file
+    """
+    # Separate requirement strings from requirement files
+    if isinstance(requirements, str):
+        requirements = {requirements}
+    else:
+        requirements = set(requirements)
+    requirement_files: Set[str] = set(
+        filter(is_configuration_file, requirements)
+    )
+    requirement_strings: Set[str] = requirements - requirement_files
+    name: str
+    return set(
+        sorted(
+            _iter_requirement_strings_required_distribution_names(
+                unique_everseen(
+                    chain(
+                        requirement_strings,
+                        *map(
+                            iter_configuration_file_requirement_strings,
+                            requirement_files,
+                        ),
+                    )
+                ),
+            ),
+            key=lambda name: name.lower(),
+        )
+    )
